tsrb2kr.Items_FriendlyFire = function(p, inflictor, source)
    -- ...unless we want to!
	if not G_RaceGametype() then return end
	if CV_FindVar("friendlyfire").value then return end

	-- check who damaged us
    local p2
    if ((inflictor and inflictor.valid) and inflictor.player)
        p2 = inflictor.player
    end
    if ((source and source.valid) and source.player)
        p2 = source.player
    end
    if not (p2 and p2.valid) then return end
	if (p == p2) then return end

    if (p.tsr.team == p2.tsr.team)	-- they're friendly
        K_DoInstashield(p)
        return false
    end
end

tsrb2kr.Items_TeamColliding = function(mo, mo2)
	-- ...unless we want to!
	if not G_RaceGametype() then return end
	if CV_FindVar("friendlyfire").value then return end

	if mo and mo.valid and mo.type == MT_PLAYER and mo2 and mo2.valid
		local accept_types = {
			MT_ORBINAUT, MT_ORBINAUT_SHIELD,
			MT_JAWZ, MT_JAWZ_DUD, MT_JAWZ_SHIELD,
			MT_BANANA, MT_BANANA_SHIELD,
			MT_EGGMANITEM, MT_EGGMANITEM_SHIELD,
			MT_SPBEXPLOSION, MT_MINEEXPLOSION,
			MT_BALLHOG
		}

		-- check if mo2 is 1 of these types, return if not
		local accepted
		for i = 1, #accept_types
			if (mo2.type == accept_types[i])
				if (mo2.type == MT_SPBEXPLOSION) -- yes eggman boxes are actually spb explosions
				and not (mo2.extravalue1) -- extravalue1 is set on real spb explosions
					return
				end
			
				accepted = true
				break
			end
		end
		if not (accepted) then return end

		if ((mo2.target and mo2.target.valid) and mo2.target.player)	-- they exist
		and (mo2.target.player.tsr.team == mo.player.tsr.team)	-- same team
		and (mo.z + mo.height >= mo2.z and mo2.z <= mo2.z + mo2.height)	-- z check
		and (mo.player != mo2.target.player) -- we can still damage ourself, though
			return false	-- this collision never happened.
		end
	end
end

local flashItems = {
	MT_EGGMANITEM, MT_EGGMANITEM_SHIELD,
	MT_BANANA, MT_BANANA_SHIELD,
	MT_ORBINAUT, MT_ORBINAUT_SHIELD,
	MT_JAWZ, MT_JAWZ_DUD, MT_JAWZ_SHIELD,
	MT_SSMINE_SHIELD, MT_SSMINE,
	MT_BALLHOG
}

tsrb2kr.Items_AddToTable = function(mo)
	if not (mo and mo.valid) then return end
	if not (G_RaceGametype()) then return end

	-- add mo to the table of stuff to do
	server.tsr_server.colouritems = $ or {}
	server.tsr_server.colouritems[#server.tsr_server.colouritems+1] = mo
end
for i = 1, #flashItems
	addHook("MobjSpawn", tsrb2kr.Items_AddToTable, flashItems[i])
end

tsrb2kr.Framework_ColourItems = function()
	if not (server.tsr_server.colouritems and #server.tsr_server.colouritems) then return end

	local i = #server.tsr_server.colouritems
	while i
		local mo = server.tsr_server.colouritems[i]

		-- check if mo is still valid
		if not (mo and mo.valid) or not mo.health
			table.remove(server.tsr_server.colouritems, i)
			i = $-1
			continue
		end

		-- now colour the item
		if (mo.target and mo.target.player.tsr.team)
			-- needs to be on a team
			if not (mo.target.player.tsr.team)
				i = $-1
				continue
			end
			
			-- spawn a marker on them if they dont have one already (not for shield items)
			local noMarker = false
			local noMarkerTypes = {MT_BANANA_SHIELD, MT_ORBINAUT_SHIELD, MT_EGGMANITEM_SHIELD, MT_JAWZ_SHIELD, MT_SSMINE}
			for item = 1, #noMarkerTypes
				if (mo.type == noMarkerTypes[item]) then noMarker = true end
			end

			if not (mo.tsr_HImarker and mo.tsr_HImarker.valid) and not noMarker
				mo.tsr_HImarker = P_SpawnMobj(mo.x, mo.y, mo.z, MT_TSR_HIGHLIGHTITEM)
				mo.tsr_HImarker.target = mo
				mo.tsr_HImarker.color = tsrb2kr.ReturnTeamColor(mo.target.player.tsr.team)
				mo.tsr_HImarker.flags2 = $|MF2_DONTDRAW
			end

			-- make sure bananas, eggboxes, jawz and ballhogs are coloured, too
			local teamSprites = {
				-- Bananas
				[MT_BANANA] = SPR_TBNA,
				[MT_BANANA_SHIELD] = SPR_TBNA,
				-- Jawz
				[MT_JAWZ] = SPR_TJWZ,
				[MT_JAWZ_DUD] = SPR_TJWZ,
				[MT_JAWZ_SHIELD] = SPR_TJWZ,
				-- Ballhog
				[MT_BALLHOG] = SPR_THOG
			}
			mo.sprite = teamSprites[mo.type] or states[mo.state].sprite

			mo.color = tsrb2kr.ReturnTeamColor(mo.target.player.tsr.team)
		else
			-- reset sprite
			mo.sprite = states[mo.state].sprite
		end

		-- go to next item in the table
		i = $-1
	end
end

addHook("MobjThinker", function(mo)
	if not G_RaceGametype() then return end
	if (mo.tsr_spbteam) then return end	-- we already got a team

	if (mo.target and mo.target.valid)	-- we need a target
	and (mo.target.player)	-- safety
		mo.tsr_spbteam = tsrb2kr.ReturnTeam(mo.target.player)
		mo.state = S_TSRSPB1	-- I guess the SPB just doesnt give a shit about mo.sprite
		-- spawn a marker on the SPB
		mo.tsr_HImarker = P_SpawnMobj(mo.x, mo.y, mo.z, MT_TSR_HIGHLIGHTITEM)
		mo.tsr_HImarker.target = mo
		mo.tsr_HImarker.color = tsrb2kr.ReturnTeamColor(mo.tsr_spbteam)
		mo.tsr_HImarker.flags2 = $|MF2_DONTDRAW
	end
end, MT_SPB)
-- add on to A_SPBChase to not target teammates (Also we colour it here too)
function A_SPBChase(actor, var1, var2)
	if CV_FindVar("friendlyfire").value
	or not G_RaceGametype()
		-- we still want to colour the spb
		if G_RaceGametype()
			if (actor.tsr_spbteam) then actor.color = tsrb2kr.ReturnTeamColor(actor.tsr_spbteam) end
		end
		-- now just run everything like normal
		super(actor, var1, var2)
		return
	end
	-- exclude players that shouldnt be targetted
	if (actor.tsr_spbteam)
		actor.color = tsrb2kr.ReturnTeamColor(actor.tsr_spbteam)
		for i = 1, #server.tsr_server.teams[actor.tsr_spbteam]
			local tp = server.tsr_server.teams[actor.tsr_spbteam][i]
			if not (tp and tp.valid) then continue end

			-- we can still fuck ourself over
			if (actor.target and actor.target.valid)
			and (actor.target.player == tp)
				continue
			end

			-- "already finished" safety check
			if (tp.exiting)
				tp.tsr_spbexiting = true
				continue
			end

			tp.exiting = 6969 -- spb cant target players with exiting set
		end
	end
	-- now run A_SPBChase with the excluded players
	super(actor, var1, var2)
	-- then remove exiting from the excluded players so nothing horrible happens
	if (actor.tsr_spbteam)
		for i = 1, #server.tsr_server.teams[actor.tsr_spbteam]
			local tp = server.tsr_server.teams[actor.tsr_spbteam][i]
			if not (tp and tp.valid) then continue end

			-- we can still fuck ourself over
			if (actor.target and actor.target.valid)
			and (actor.target.player == tp)
				continue
			end

			-- "already finished" safety check
			if (tp.tsr_spbexiting)
				tp.tsr_spbexiting = nil
				continue
			end

			tp.exiting = 0 -- spb cant target players with exiting set
		end
	end
end

tsrb2kr.Framework_JawzTargeting = function(mo)
	if CV_FindVar("friendlyfire").value then return end

	if (mo.player.kartstuff[k_itemtype] == KITEM_JAWZ and mo.player.kartstuff[k_itemheld])
		-- btech jawz targeting with teams involved
		local targ
		local best = -1

		for p in players.iterate
			-- check player stuff first
			if not (p and p.mo and p.mo.health) then continue end -- needs to be alive
			if (p.spectator) then continue end -- cant be a spectator
			if (p == mo.player) then continue end -- dont target ourselves
			if (p.kartstuff[k_hyudorotimer]) then continue end -- dont target invisible players
			if (tsrb2kr.ReturnTeam(mo.player) == tsrb2kr.ReturnTeam(p)) then continue end -- dont target teammates

			-- get angle & distance
			local a = mo.angle - R_PointToAngle2(mo.x, mo.y, p.mo.x, p.mo.y)
			if (a > ANGLE_180) then a = InvAngle(a) end
			local d = P_AproxDistance(p.mo.x - (mo.x + mo.momx), p.mo.y - (mo.y + mo.momy))

			-- now let's find a target!
			if (a > ANGLE_67h) then continue end -- dont target players behind you
			if (p.kartstuff[k_position] >= mo.player.kartstuff[k_position]) then continue end -- target players in a higher position

			if (best == -1 or p.kartstuff[k_position] > best)
				targ = p
				best = p.kartstuff[k_position]
			end
		end

		-- now override the target
		if targ -- check if we have a target
			if (mo.player.kartstuff[k_lastjawztarget] ~= #targ) then S_StartSound(targ.mo, sfx_s3k89) end
			mo.player.kartstuff[k_lastjawztarget] = #targ
			mo.player.kartstuff[k_jawztargetdelay] = 5
		end
	end
end
-- make sure it's not targetting a teammate
addHook("MobjThinker", function(mo)
	if not G_RaceGametype() then return end

	if (mo and mo.valid) -- paranoia
		if (mo.target and mo.target.valid)
		and (mo.tracer and mo.tracer.valid)
			if (tsrb2kr.ReturnTeam(mo.target.player) == tsrb2kr.ReturnTeam(mo.tracer.player))
			and not CV_FindVar("friendlyfire").value
				mo.tracer = nil
			end
		end
	end
end, MT_JAWZ)