-- everything slingshot related
tsrb2kr.Slingshots_RemoveTrail = function(p)
	if not (p.tsr.trailmos and #p.tsr.trailmos) then return end
	
	for i = #p.tsr.trailmobjs, 1, -1
		local trailmo = p.tsr.trailmobjs[i]
		if not (trailmo and trailmo.valid) then continue end
		
		P_RemoveMobj(trailmo)
	end
end
addHook("PlayerQuit", tsrb2kr.Slingshots_RemoveTrail)

tsrb2kr.Slingshots_SpawnTrail = function(p)
	local mo = p.mo

	-- handle trail table
	p.tsr.trailmos = $ or {}
	for i = #p.tsr.trailmos, 1, -1
		local trailmo = p.tsr.trailmos[i]
		if not (trailmo and trailmo.valid)
			table.remove(p.tsr.trailmos, i)
			i = $-1
		end
	end
	
	if (p.kartstuff[k_offroad]) then return end	-- no trails in offroad
	if (p.kartstuff[k_spinouttimer]) then return end	-- no trails while spun out
	if (p.exiting) then return end	-- no trails after finishing

	-- check if we're the lead player
	local team = server.tsr_server.teams[p.tsr.team]
	
	-- if we HAD a teammate before, still remove our trail
	if (#team == 1) then tsrb2kr.Slingshots_RemoveTrail(p) return end
	
	-- all around me are familiar faces
	for i = 1, #team
		local p2 = team[i]
		if (p == p2) then continue end
		if (p.kartstuff[k_position] < p2.kartstuff[k_position]) then continue end
		
		-- someone in our team has a lower place than us, so we defo aren't the lead
		return
	end
	
	if not (mo and mo.valid) then return end

	-- now that we've established that we're the lead, let's see if we can spawn the trail
	if (P_IsObjectOnGround(mo))	-- we're on the floor
	and (R_PointToDist2(0, 0, mo.momx, mo.momy) > K_GetKartSpeed(p)*2/5)	-- we're fast enough
		-- we are now elligible for spawning boost trail.
		-- amazing!!!!!
		if not (leveltime%4)
			-- set the delay before a trail should be removed
			local delay = max(8, 36 - (mapheaderinfo[gamemap].numlaps*7))
			if mapheaderinfo[gamemap].levelflags & LF_SECTIONRACE
			or delay > 25
				delay = 25
			end

			-- the size of our trail is determined by our stats
			local trailsize = (96 - (p.kartspeed-1)*2 - (p.kartweight-1)*2)
			trailsize = $*mapobjectscale/96	-- base size of 96

			local trailmo = P_SpawnMobj(mo.x, mo.y, mo.z, MT_TSR_BOOST)
			trailmo.state = S_THOK
			trailmo.sprite = SPR_TSRT
			trailmo.color = tsrb2kr.ReturnTeamColor(p.tsr.team)	-- sure that's cute
			trailmo.target = mo		-- used to let other players know who spawned it, so whether or not they'll benefit from it
			trailmo.scale = trailsize
			trailmo.frame = A|FF_FULLBRIGHT|FF_PAPERSPRITE
			trailmo.angle = R_PointToAngle2(0, 0, mo.momx, mo.momy) + ANG1*90
			trailmo.tics = delay*TICRATE
			trailmo.savecolor = trailmo.color
			tsrb2kr.FlipFromObject(trailmo, mo)
			table.insert(p.tsr.trailmos, trailmo)
			
			-- the blessing of 1.2
			local drawFlags = {MFE_DRAWONLYFORP1, MFE_DRAWONLYFORP2, MFE_DRAWONLYFORP3, MFE_DRAWONLYFORP4}
			for i = 0, (splitscreen) -- go through displayplayers
				if not displayplayers[i] then break end

				if (p.tsr.team == displayplayers[i].tsr.team)
					trailmo.eflags = $|drawFlags[i+1]
				end
			end
			-- somehow this isnt supposed to be displayed at ALL for any displayplayers, so dont draw it
			if not (trailmo.eflags & (MFE_DRAWONLYFORP1|MFE_DRAWONLYFORP2|MFE_DRAWONLYFORP3|MFE_DRAWONLYFORP4))
				trailmo.flags2 = $|MF2_DONTDRAW
			end
		end
	end
end

tsrb2kr.Slingshots_TrailCollide = function(trail, mo)
	if not (trail and trail.valid) then return end
	if not (mo and mo.valid) then return end
	
	if (mo.type != MT_PLAYER) then return end	-- must be a player
	if (trail.target == mo) then return end	-- no self boosting
	if (trail == mo.tsr_lasttrailcollide) then return end	-- we already hit this last frame
	
	local p = mo.player
	if not (trail.target and trail.target.valid) then return end
	local p2 = trail.target.player
	if (p.tsr.team != p2.tsr.team) then return end	-- not on the same team? foh
	
	if (mo.z + mo.height >= trail.z)
	and (mo.z <= trail.z + trail.height)	-- z checks (...)
		mo.tsr_lasttrailcollide = trail
		mo.tsr_slingshotout = 10 + max(0, 20 - (p.speed)>>FRACBITS)	-- # of tics we have to step out for to gain the boost
		-- ^ the slower you go, the more lenient it is. But you won't be able to build slingshot until you've reached a good speed.
		
		mo.tsr_slingshottarget = trail.target.player	-- used to make the trail flash
	end
end
addHook("MobjCollide", tsrb2kr.Slingshots_TrailCollide, MT_TSR_BOOST)
addHook("MobjMoveCollide", tsrb2kr.Slingshots_TrailCollide, MT_TSR_BOOST)

tsrb2kr.Slingshots_ChargeBoost = function(p)
	local mo = p.mo
	if not (mo and mo.valid) then return end

	mo.tsr_slingshot = $ or 0	-- how many boosts have you hit thus far?
	mo.tsr_slingshotout = $ or 0	-- timer to determine if you've been out of the trail for long enough to initiate the slingshot
	mo.tsr_slingshotprevlevel = $ or 0	-- debug

	if (mo.tsr_slingshotout)
		-- we are on the slingshot: if possible, highlight the trail if we are a displayplayer
		if (mo.tsr_slingshottarget and mo.tsr_slingshottarget.valid) and mo.tsr_slingshottarget.tsr.trailmos
		and (tsrb2kr.IsDisplayPlayer(p) != false)
			for k, v in ipairs(mo.tsr_slingshottarget.tsr.trailmos)
				if not (v and v.valid) then continue end
				
				v.color = SKINCOLOR_WHITE
			end
		end

		-- we... aren't moving... what
		local spd = R_PointToDist2(0, 0, mo.momx, mo.momy)
		if (spd < 2*mo.scale)
			return
		elseif (spd > K_GetKartSpeed(p)*2/5)	-- only start building slingshot once you're fast enough
		and P_IsObjectOnGround(mo)	-- still, we shouldn't build slingshot if we aren't on the floor
			mo.tsr_slingshot = $+1
		end

		-- we are in the slingshot trail.
		-- grant us increased acceleration

		local speedboost = min(FRACUNIT/8, (mo.tsr_slingshot)*FRACUNIT*8/1000)
		p.kartstuff[k_speedboost] = max($, speedboost)	-- max: 12.5% speed boost after about 4 seconds
		p.kartstuff[k_accelboost] = max($, FRACUNIT/4)	-- 125% accel

		if (mo.tsr_slingshot > 20)	-- after at least 20 tics
			local slingshot = mo.tsr_slingshot-20
			local levels = {10, 35, 70}
			
			-- get level
			for i = 1, #levels do
				if (slingshot >= levels[i])
					mo.tsr_slingshotlevel = i
				end
			end

			-- old behavior
			--mo.tsr_slingshotlevel = min(3, mo.tsr_slingshot / 25)

			-- flash every now and then depending on your level
			if (mo.tsr_slingshotlevel)
				-- level up spark
				if (mo.tsr_slingshotlevel > mo.tsr_slingshotprevlevel)
					--CONS_Printf(p, "Slingshot: level "..mo.tsr_slingshotlevel)
					mo.tsr_slingshotprevlevel = mo.tsr_slingshotlevel
					-- visual effect:
					local colours = {SKINCOLOR_BLUE, SKINCOLOR_PURPLE, SKINCOLOR_RED}

					local b = P_SpawnMobj(mo.x, mo.y, mo.z, MT_TSR_SLINGSHOTLEVEL)
					b.color = colours[mo.tsr_slingshotlevel]
					b.scale = mo.scale
					b.destscale = mo.scale*2
					b.scalespeed = mo.scale/3
					b.target = mo
					b.angle = mo.angle + ANG1*90
					S_StartSoundAtVolume(mo, sfx_s3ka2, 192)
					tsrb2kr.FlipFromObject(b, mo)
				end

				local divide = 6 / mo.tsr_slingshotlevel
				if not (leveltime%divide)
					mo.colorized = true
				else
					mo.colorized = false
				end
			end
		end
		
		if (P_IsObjectOnGround(mo))	-- do not deplete slingshot timer in mid-air
			-- do NOT let this go during a boost
			if (mo.tsr_slingshotout == 2)
			and (p.kartstuff[k_sneakertimer]
			or p.kartstuff[k_startboost]
			or p.kartstuff[k_driftboost])
				return
			end
			mo.tsr_slingshotout = $-1
		end

		if (mo.tsr_slingshotout == 1)	-- oh shit, time to BOOST
			p.kartstuff[k_accelboost] = 0

			local s = mo.tsr_slingshottarget.kartspeed
			local w = mo.tsr_slingshottarget.kartweight

			-- at 9, 1, boostadd is 13
			-- at 5, 5, boostadd is 5
			-- at 1, 9, boostadd is -5

			local boostadd = 5 - w + s

			if (mo.tsr_slingshotlevel)
			and not (p.kartstuff[k_spinouttimer])	-- spun out, don't boost.
				local boost_levels = {18, 24, 44}

				p.kartstuff[k_driftboost] = boost_levels[mo.tsr_slingshotlevel] + boostadd
				tsrb2kr.FillUltimate(p, 20 + 12*mo.tsr_slingshotlevel, "SLINGSHOT!", {tsrb2kr.ColorToMap(tsrb2kr.ReturnTeamColor(p.tsr.team), true)}, sfx_ncitem)
				S_StartSound(mo, sfx_s23c)
			end

			mo.tsr_slingshotlevel = 0
			mo.tsr_slingshot = 0
			mo.tsr_slingshotout = 0
			mo.tsr_slingshotprevlevel = 0

			-- we are no longer on the slingshot trail, revert it back to normal colours
			if (mo.tsr_slingshottarget and mo.tsr_slingshottarget.valid) and mo.tsr_slingshottarget.tsr.trailmos
			and (tsrb2kr.IsDisplayPlayer(p) != false)
				for k, v in ipairs(mo.tsr_slingshottarget.tsr.trailmos)
					if (v and v.valid)
						--print(v.hprev.color.."|"..v.hprev.savecolor)
						--v.hprev.color = v.hprev.savecolor
						v.color = v.savecolor
					end
				end
			end
		end
	end
end

tsrb2kr.Slingshots_PlayerDeath = function(mo)
	if (mo and mo.valid) 
	and (mo.tsr_slingshottarget and mo.tsr_slingshottarget.valid)
		for k, v in ipairs(mo.tsr_slingshottarget.tsr.trailmos)
			if not (v and v.valid) then continue end
			
			--v.hprev.color = v.hprev.savecolor or 0
			v.color = v.savecolor or 0
		end
	end
end

tsrb2kr.Framework_HandleSlingshots = function(p)
	if not (p.tsr.team) then return end
	tsrb2kr.Slingshots_SpawnTrail(p)
	tsrb2kr.Slingshots_ChargeBoost(p)
end