-- spectators
local BASEVIDWIDTH = 320
local SPECTEXTWIDTH = 6
tsrb2kr.HUD_DrawSpectators = function(v)
    if not ((netgame) and G_GametypeHasSpectators()) then return end

    local length = 0
    local height = 178
    local totallength = 0
    local templength = -8
    local dupadjust = v.width() / v.dupx()
    local duptweak = (dupadjust - BASEVIDWIDTH)/2

    for p in players.iterate
        if not (p.spectator) then continue end
        totallength = $+p.name:len()*SPECTEXTWIDTH + (SPECTEXTWIDTH*2)
    end

    length = $ - (leveltime % (totallength + dupadjust+SPECTEXTWIDTH))
    length = $ + dupadjust

    for p in players.iterate
        if not (p.spectator) then continue end

        local le = p.name:len()*SPECTEXTWIDTH + (SPECTEXTWIDTH*2)
        if (length >= -le)
            v.drawString(length - duptweak, height, p.name, V_TRANSLUCENT|V_ALLOWLOWERCASE|V_6WIDTHSPACE, "thin")
        end

        length = $+le
        if (length >= dupadjust+SPECTEXTWIDTH) then break end
    end
end

hud.add(function(v, p, c)
	if not G_RaceGametype()
		hud.enable("rankings")
		return
	end
	
	if (leveltime >= 106)	-- we only need team rankings after teams are assigned
	or (server.tsr_server and server.tsr_server.tourneyround)	-- they're instantly assigned
		hud.disable("rankings")
		v.fadeScreen(0xFF00, 16)	-- fade the screen
		
		-- this will be useful
		local dupadjust = (v.width()/v.dupx())
		local duptweak = (dupadjust - 320)/2	-- BASEDVIDWIDTH = 320

		-- store highlight colour here for use in the hud throughout
		local hilicol
		if (CV_FindVar("menuhighlight").value)
			hilicol = CV_FindVar("menuhighlight").value
		elseif (modeattacking)
			hilicol = V_ORANGEMAP
		else
			hilicol = ((gametype == GT_RACE) and V_SKYMAP or V_REDMAP)
		end

		-- this is TSR
		v.drawString(4, 188, "TSR", hilicol|V_SNAPTOBOTTOM|V_SNAPTOLEFT)

		-- draw lap count and game speed
		v.drawString(64, 8, "LAP COUNT", 0, "center")
		v.drawString(64, 16, (numlaps), hilicol, "center")
		v.drawString(256, 8, "GAME SPEED", 0, "center")
		v.drawString(256, 16, CV_FindVar("kartspeed").string, hilicol, "center")

		-- draw line between gametypeinfo and teams
		v.drawFill(1-duptweak, 26, dupadjust-2, 1, 0)	-- horizontal
		v.drawFill(160, 26, 1, 151, 0)	-- vertical
		tsrb2kr.HUD_DrawSpectators(v)	-- spectators

		-- draw players
		local tsr_rankings = {}
		for p in players.iterate
			if (p and p.valid) and not p.spectator
				table.insert(tsr_rankings, p)
			end
		end
		table.sort(tsr_rankings, function(a, b) 
			-- if they've finished, they should be prioritized on their time
			if (a.exiting)
				return a.realtime < b.realtime
			end
			-- otherwise go on position
			return a.kartstuff[k_position] < b.kartstuff[k_position] 
		end)
		if not #tsr_rankings then return end

		for i = 1, #tsr_rankings
			local p = tsr_rankings[i]
			local hy = 28 + (i-1)*10
			
			-- player ping:
			if (p ~= server)  -- the server doesnt have a ping
			and (netgame)	-- we obviously need to be in a netgame too
				-- my hopes with writing this code is that it's so bad that I'm never allowed to write ping related code ever again
				local ping = p.ping	-- bless 1.3
				-- get the ping patch we'll use
				local ping_gfxnum, ping_values = 1, {76, 137, 256, 500}
				for i = 1, #ping_values
					if (ping >= ping_values[i]) then ping_gfxnum = i+1 end
				end
				-- sadly p.cmd.latency caps at 11 so it cant go any higher without hacks so huge that I cant be assed
				v.draw(3, hy, v.cachePatch("TSRPING"..ping_gfxnum))
			end
			-- rank:
			v.drawString(29, hy+1, p.kartstuff[k_position], 0, "right")
			-- draw skin icon
			local isConsolePlayer = (not (splitscreen) and (p == consoleplayer)) and true or false
			local pp = skins[p.mo.skin].facerank
			local colormap = p.mo.colorized and TC_RAINBOW or p.mo.skin
			v.drawScaled(30<<FRACBITS, (hy+1)<<FRACBITS, FRACUNIT/2, v.cachePatch(pp), 0, v.getColormap(colormap, p.mo.color))
			if (isConsolePlayer)
				local pp = "K_CHILI"..(leveltime%32/4)+1
				v.drawScaled(30<<FRACBITS, (hy+1)<<FRACBITS, FRACUNIT/2, v.cachePatch(pp)) 
			end
			-- name: c[p.tsr_team]..p.name
			local c = {
				tsrb2kr.ColorToMap(tsrb2kr.ReturnTeamColor(1)),
				tsrb2kr.ColorToMap(tsrb2kr.ReturnTeamColor(2)),
				tsrb2kr.ColorToMap(tsrb2kr.ReturnTeamColor(3)),
				tsrb2kr.ColorToMap(tsrb2kr.ReturnTeamColor(4)),
				tsrb2kr.ColorToMap(tsrb2kr.ReturnTeamColor(5))
			}
			local index = p.tsr.team or 6
			v.drawString(39, hy, c[p.tsr.team]..p.name, V_6WIDTHSPACE|V_ALLOWLOWERCASE, "thin")
			-- points:
			local points = tsrb2kr.ReturnPoints(p)
			v.drawString(157, hy, points, (p.exiting and hilicol or 0)|V_6WIDTHSPACE, "thin-right")
		end

		-- draw team scores
		local sortedteams = server.tsr_server.sortedteams
		if not (sortedteams) then return end
		if not (#sortedteams) then return end	-- need to have valid teams
		
		for i = 1, #sortedteams
			local index = sortedteams[i][1]
			local hx = 196 + (28*(index-1))
			local hy = 177

			-- team position
			local pp = "K_POSN"..sortedteams[i][3].."0"
			v.drawScaled(hx<<FRACBITS, hy<<FRACBITS, FRACUNIT/2, v.cachePatch(pp), V_HUDTRANSHALF)	-- team ranking
			-- team score bar
			local cb = {120, 5, 15, 21, 31, 22, 14, 32, 179, 202, 34, 36, 52, 54, 61, 124, 146, 151, 148, 152, 134, 127, 125, 126, 136, 137, 99, 85, 87, 127, 90, 89, 83, 88, 92, 94, 155, 85, 72, 68, 97, 113, 252, 119, 61, 106, 102, 113, 176, 107, 84, 114, 160, 168, 176, 178, 180, 182, 169, 163, 165, 167, 184, 187, 162, 186, 167, 220, 216, 214, 216, 210, 217, 215, 218, 11, 201, 202, 113, 203, 53, 205, 215, 227, 230, 235, 248, 249, 196, 192, 204, 192, 193, 126, 10, 113, 248, 254, 148, 122}
			v.drawFill(hx-20, hy-136, 18, 105, 24)
			-- the points
			local p = sortedteams[i][2]
			v.drawFill(hx-19, hy-135+(103-(p*2)), 16, p*2, cb[tsrb2kr.ReturnTeamColor(index)])
			-- point borders + icons
			-- sort team players by position
			local sorted_team = {}
			for i = 1, #server.tsr_server.teams[index]
				local tp = server.tsr_server.teams[index][i]
				if not (tp and tp.valid) then continue end
				
				table.insert(sorted_team, tp)
			end
			table.sort(sorted_team, function(a, b) return a.kartstuff[k_position] > b.kartstuff[k_position] end)
			-- now draw their borders
			local added_points = 0
			for i = 1, #sorted_team
				local tp = sorted_team[i]
				local isConsolePlayer = (not (splitscreen) and (tp == consoleplayer)) and true or false
				added_points = $+tsrb2kr.ReturnPoints(tp)
				
				v.drawFill(hx-20, hy-135+(103-(added_points*2)), 18, 1, 24)
				if (tp.mo and tp.mo.valid)
					v.drawScaled((hx-28)<<FRACBITS, (hy-135+(103-(added_points*2))-4)<<FRACBITS, FRACUNIT/2, v.cachePatch(skins[tp.mo.skin].facerank), 0, v.getColormap(tp.mo.skin, tp.skincolor))
					if (isConsolePlayer)
						local pp = "K_CHILI"..(leveltime%32/4)+1
						v.drawScaled((hx-28)<<FRACBITS, (hy-135+(103-(added_points*2))-4)<<FRACBITS, FRACUNIT/2, v.cachePatch(pp)) 
					end
				end
			end
			-- points string
			local w = v.stringWidth(p, V_6WIDTHSPACE, "thin")
			v.drawString(hx-11-(w/2), hy-42, p, V_6WIDTHSPACE, "thin")
			-- team name
			local teamname = tsrb2kr.ReturnTeamName(index)
			if (teamname:len() > 6)
				teamname = $:sub(1, 5)..".."
			end
			
			local c = {
				tsrb2kr.ColorToMap(tsrb2kr.ReturnTeamColor(1)),
				tsrb2kr.ColorToMap(tsrb2kr.ReturnTeamColor(2)),
				tsrb2kr.ColorToMap(tsrb2kr.ReturnTeamColor(3)),
				tsrb2kr.ColorToMap(tsrb2kr.ReturnTeamColor(4)),
				tsrb2kr.ColorToMap(tsrb2kr.ReturnTeamColor(5))
			}
			local w = v.stringWidth(c[index]..teamname, V_ALLOWLOWERCASE|V_6WIDTHSPACE, "thin")
			v.drawString(hx-11-(w/2), hy-148, c[index]..teamname, V_ALLOWLOWERCASE|V_6WIDTHSPACE, "thin")
		end
	else
		hud.enable("rankings")
	end
end, "scores")